@@ -1,20 +1,21 @@
 # mpatch.py - Python implementation of mpatch.c
 #
-# Copyright 2009 Matt Mackall <mpm@selenic.com> and others
+# Copyright 2009 Olivia Mackall <olivia@selenic.com> and others
 #
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2 or any later version.
 
-from __future__ import absolute_import
 
+import io
 import struct
 
-from .. import pycompat
-stringio = pycompat.bytesio
+
+stringio = io.BytesIO
+
 
 class mpatchError(Exception):
-    """error raised when a delta cannot be decoded
-    """
+    """error raised when a delta cannot be decoded"""
+
 
 # This attempts to apply a series of patches in time proportional to
 # the total size of the patches, rather than patches * len(text). This
@@ -26,16 +27,18 @@ class mpatchError(Exception):
 # mmap and simply use memmove. This avoids creating a bunch of large
 # temporary string buffers.
 
-def _pull(dst, src, l): # pull l bytes from src
+
+def _pull(dst, src, l):  # pull l bytes from src
     while l:
         f = src.pop()
-        if f[0] > l: # do we need to split?
+        if f[0] > l:  # do we need to split?
             src.append((f[0] - l, f[1] + l))
             dst.append((l, f[1]))
             return
         dst.append(f)
         l -= f[0]
 
+
 def _move(m, dest, src, count):
     """move count bytes from src to dest
 
@@ -46,6 +49,7 @@ def _move(m, dest, src, count):
     m.seek(dest)
     m.write(buf)
 
+
 def _collect(m, buf, list):
     start = buf
     for l, p in reversed(list):
@@ -53,6 +57,7 @@ def _collect(m, buf, list):
         buf += l
     return (buf - start, start)
 
+
 def patches(a, bins):
     if not bins:
         return a
@@ -60,7 +65,7 @@ def patches(a, bins):
     plens = [len(x) for x in bins]
     pl = sum(plens)
     bl = len(a) + pl
-    tl = bl + bl + pl # enough for the patches and two working texts
+    tl = bl + bl + pl  # enough for the patches and two working texts
     b1, b2 = 0, bl
 
     if not tl:
@@ -90,29 +95,30 @@ def patches(a, bins):
         while pos < end:
             m.seek(pos)
             try:
-                p1, p2, l = struct.unpack(">lll", m.read(12))
+                p1, p2, l = struct.unpack(b">lll", m.read(12))
             except struct.error:
-                raise mpatchError("patch cannot be decoded")
-            _pull(new, frags, p1 - last) # what didn't change
-            _pull([], frags, p2 - p1)    # what got deleted
-            new.append((l, pos + 12))   # what got added
+                raise mpatchError(b"patch cannot be decoded")
+            _pull(new, frags, p1 - last)  # what didn't change
+            _pull([], frags, p2 - p1)  # what got deleted
+            new.append((l, pos + 12))  # what got added
             pos += l + 12
             last = p2
-        frags.extend(reversed(new))     # what was left at the end
+        frags.extend(reversed(new))  # what was left at the end
 
     t = _collect(m, b2, frags)
 
     m.seek(t[1])
     return m.read(t[0])
 
+
 def patchedsize(orig, delta):
     outlen, last, bin = 0, 0, 0
     binend = len(delta)
     data = 12
 
     while data <= binend:
-        decode = delta[bin:bin + 12]
-        start, end, length = struct.unpack(">lll", decode)
+        decode = delta[bin : bin + 12]
+        start, end, length = struct.unpack(b">lll", decode)
         if start > end:
             break
         bin = data + length
@@ -122,7 +128,7 @@ def patchedsize(orig, delta):
         outlen += length
 
     if bin != binend:
-        raise mpatchError("patch cannot be decoded")
+        raise mpatchError(b"patch cannot be decoded")
 
     outlen += orig - last
     return outlen
